import stringcase
import numbers
import inspect
from spb.orm.type import String, ID

class Query(object):

    @classmethod
    def get(cls, model, optional):
        where = cls._where_message(cls, model=model, optional=optional)
        query_id = cls._query_id(cls, model=model)
        attrs = cls._model_attrs_to_query(cls, model=model)
        return cls._make_query(cls, query_id=query_id, where=where, attrs=attrs)

    @classmethod
    def mutation(cls, model, optional):
        query_id = cls._mutation_query_id(cls, model=model)
        attrs = cls._model_attrs_to_query(cls, model=model)
        values = cls._model_attrs_to_values(
            cls, model=model, optional=optional)
        return cls._make_mutation(cls, query_id=query_id, values=values, attrs=attrs)

    @classmethod
    def delete(cls, model, optional):
        pass

    def _model_attrs_to_values(self, model, optional):
        query = Query()
        immutable_attrs = model._immutables
        attrs = model.get_attr_names().keys()
        mutable_attrs = [attr for attr in attrs if attr not in immutable_attrs]
        for key, value in model._attrs.items():
            if isinstance(value, ID) and not key in mutable_attrs:
                mutable_attrs.append(key)
        return query._params(mode='MUTATION', model=model, used_attrs=mutable_attrs, optional=optional)

    def _where_message(self, model, optional):
        query = Query()
        filterable_attrs = model._filterables
        return query._params(model=model, used_attrs=filterable_attrs, optional=optional)

    def _params(self, model, used_attrs, optional, mode='QUERY'):
        if used_attrs is None:
            return None
        attr_classes = model._attrs_classes
        temp = {}
        for attr in used_attrs:
            name = model.get_attr_names().get(attr, None)
            if name is not None:
                value = model.attribute_values[name]
                if value is not None:
                    temp.update(attr_classes[name]._type_to_query(attr, value))

        if mode == 'QUERY':
            for key, value in optional.items():
                if value is not None:
                    temp[key] = value if str(value).isdigit() else f'"{str(value)}"'

        elif mode == 'MUTATION':
            for key, value in optional.items():
                if value is not None and key is not 'page' and key is not 'pageSize':
                    temp[key] = value if str(value).isdigit() else f'"{str(value)}"'

        params = []
        for key, value in temp.items():
            params.append(f'{key}:{value}')

        if len(params) > 0:
            return f'({",".join(params)})'
        else:
            return None

    def _query_id(self, model):
        return model.get_resource_name()

    def _mutation_query_id(self, model):
        if model.id is None:
            return stringcase.camelcase('create_' + model.get_resource_name())
        else:
            return stringcase.camelcase('update_' + model.get_resource_name())

    def _model_attrs_to_query(self, model):
        model_attrs = model.get_attr_names()
        # TODO: add subquery code
        return ' '.join(model_attrs.keys())

    def _make_query(self, query_id, where=None, attrs=None):
        query_str = '{' + query_id
        query_str += where if where is not None else ''
        query_str += '{' + attrs + '}}'
        return query_str

    def _make_mutation(self, query_id, values, attrs):
        mutation_str = 'mutation {' + query_id + values + '{' + attrs + '}}'
        return mutation_str



import spb

from .utils import *
from .query import Query


class Manager(object):  # pylint: disable=R0205
    ''' Data mapper interface (generic repository) for models '''

    def __init__(self, model, type_check=True):
        self.model = model
        self.type_check = type_check
        self.session = spb._get_default_session()

    def _get_model_object(self, options):
        optionUUID = options.MODEL_UUID if hasattr(options, 'MODEL_UUID') else False
        if optionUUID and optionUUID == self.model.MODEL_UUID:
            return options
        else:
            return self.model(options)

    def query(self, options, optional):
        '''Convert model to graphql query string'''
        model = self._get_model_object(options)
        query = Query.get(model, optional)
        result = self.session.execute(self.model, query)
        return result

    def mutation(self, options, optional={}):
        '''Convert model to graphql mutation string'''
        query = None
        model = self._get_model_object(options)
        result = None
        query = Query.mutation(model, optional)
        result = self.session.execute(self.model, query)
        return result

    def get_query(self, options, optional):
        model = self._get_model_object(options)
        return Query.get(model, optional)

    def get_mutation(self, options, optional):
        model = self._get_model_object(options)
        return Query.mutation(model, optional)

    def delete(self, options, optional):
        model = self._get_model_object(options)
        query = Query.delete(model, optional)
        result = self.session.execute(self.model, query)
        return result



import json
import re
from urllib.parse import urlparse


def attrs(obj):
    ''' Return attribute values dictionary for an object '''
    return dict(i for i in vars(obj).items() if i[0][0] != '_')


def copy_attrs(obj, remove=None):
    ''' Copy attribute values for an object '''
    if remove is None:
        remove = []
    return dict(i for i in attrs(obj).items() if i[0] not in remove)

def is_json(value):
    try:
        json.loads(value)
    except (ValueError, TypeError):
        return False
    return True


def is_data_url(value):
    pattern = re.compile("data:([\\w\\/\\+]+);(charset=[\\w-]+|base64).*,([a-zA-Z0-9+/]+={0,2})")
    return pattern.match(value)

def is_url(value):
    try:
        urlparse(value)
        return True
    except:
        return False



import json
from spb.orm.type import Type
from spb.orm import Model
from spb.orm import AttributeModel
from spb.orm import ListAttribute
from spb.orm.type import String, ID, Number, Object, JsonObject


class Stats(AttributeModel, ListAttribute):
    def __init__(self, *args, **kwargs):
        self.name = kwargs['name'] if 'name' in kwargs else None
        self.count = kwargs['count'] if 'count' in kwargs else None


class Label(Model):
    MODEL_UUID = '16be2af8-958b-11ea-bb37-0242ac130002'
    RESOURCE_NAME = 'labels'

    # id
    id = ID(property_name='id', immutable=True, filterable=True)
    project_id = ID(property_name='projectId', filterable=True)

    # basic info
    status = String(property_name='status', immutable=True)
    stats = Object(property_name='stats', immutable=True, express=Stats)

    # For assets
    dataset = String(property_name='dataset', immutable=True, filterable=True)
    data_key = String(property_name='dataKey', immutable=True, filterable=True)
    data_url = String(property_name='dataUrl', immutable=True)

    # label datas
    result = JsonObject(property_name='result')

    def toJson(self):
        return json.dumps({
            'id': self.id,
            'project_id': self.project_id,
            'status': self.status,
            'stats': [stat.get_datas(stat) for stat in self.stats] if self.stats is not None else None,
            'dataset': self.dataset,
            'data_key': self.data_key,
            'result': self.result
        }, indent=4)




import mimetypes
import re
import textwrap

try:
    from base64 import decodebytes as decode64
    from base64 import encodebytes as encode64
    BYTES = True
except ImportError:
    from base64 import decodestring as decode64
    from base64 import encodestring as encode64
    BYTES = False

try:
    from urllib.parse import quote, unquote
except ImportError:
    from urllib import quote, unquote


from .exceptions import InvalidCharset, InvalidDataURI, InvalidMimeType


MIMETYPE_REGEX = r'[\w]+\/[\w\-\+\.]+'
_MIMETYPE_RE = re.compile('^{}$'.format(MIMETYPE_REGEX))

CHARSET_REGEX = r'[\w\-\+\.]+'
_CHARSET_RE = re.compile('^{}$'.format(CHARSET_REGEX))

DATA_URI_REGEX = (
    r'data:' +
    r'(?P<mimetype>{})?'.format(MIMETYPE_REGEX) +
    r"(?:\;name\=(?P<name>[\w\.\-%!*'~\(\)]+))?" +
    r'(?:\;charset\=(?P<charset>{}))?'.format(CHARSET_REGEX) +
    r'(?P<base64>\;base64)?' +
    r',(?P<data>.*)')
_DATA_URI_RE = re.compile(r'^{}$'.format(DATA_URI_REGEX), re.DOTALL)


class DataURI(str):

    @classmethod
    def make(cls, mimetype, charset, base64, data):
        parts = ['data:']
        if mimetype is not None:
            if not _MIMETYPE_RE.match(mimetype):
                raise InvalidMimeType("Invalid mimetype: %r" % mimetype)
            parts.append(mimetype)
        if charset is not None:
            if not _CHARSET_RE.match(charset):
                raise InvalidCharset("Invalid charset: %r" % charset)
            parts.extend([';charset=', charset])
        if base64:
            parts.append(';base64')
            if BYTES:
                _charset = charset or 'utf-8'
                if isinstance(data, bytes):
                    _data = data
                else:
                    _data = bytes(data, _charset)
                encoded_data = encode64(_data).decode(_charset).strip()
            else:
                encoded_data = encode64(data).strip()
        else:
            encoded_data = quote(data)
        parts.extend([',', encoded_data])
        return cls(''.join(parts))

    @classmethod
    def from_file(cls, filename, charset=None, base64=True):
        mimetype, _ = mimetypes.guess_type(filename, strict=False)
        with open(filename, 'rb') as fp:
            data = fp.read()

        return cls.make(mimetype, charset, base64, data)

    def __new__(cls, *args, **kwargs):
        uri = super(DataURI, cls).__new__(cls, *args, **kwargs)
        uri._parse  # Trigger any ValueErrors on instantiation.
        return uri

    def __repr__(self):
        return 'DataURI(%s)' % (super(DataURI, self).__repr__(),)

    def wrap(self, width=76):
        return '\n'.join(textwrap.wrap(self, width))

    @property
    def mimetype(self):
        return self._parse[0]

    @property
    def name(self):
        name = self._parse[1]
        if name is not None:
            return unquote(name)
        return name

    @property
    def charset(self):
        return self._parse[2]

    @property
    def is_base64(self):
        return self._parse[3]

    @property
    def data(self):
        return self._parse[4]

    @property
    def text(self):
        if self.charset is None:
            raise InvalidCharset("DataURI has no encoding set.")

        return self.data.decode(self.charset)

    @property
    def _parse(self):
        match = _DATA_URI_RE.match(self)
        if not match:
            raise InvalidDataURI("Not a valid data URI: %r" % self)
        mimetype = match.group('mimetype') or None
        name = match.group('name') or None
        charset = match.group('charset') or None

        if match.group('base64'):
            if BYTES:
                _charset = charset or 'utf-8'
                _data = bytes(match.group('data'), _charset)
                data = decode64(_data)
            else:
                data = decode64(match.group('data'))
        else:
            data = unquote(match.group('data'))

        return mimetype, name, charset, bool(match.group('base64')), data



import configparser
import click
import rich
import rich.table
import rich.console
import os

import spb
from spb.cli_core.helper import Helper
from spb.session import Session

console = rich.console.Console()
helper = Helper()

@click.group()
@click.version_option(version=spb.__version__, message='%(version)s')
def cli():
    pass


@cli.command()
@click.option('--profile_name', prompt='Profile Name', default='default')
@click.option('--account_name', prompt='Account Name')
@click.option('--access_key', prompt='Access Key')
def config(profile_name, account_name, access_key):
    while profile_name != 'default':
        access_key = click.prompt('Authentication failed. Please try again')

    # Repeat getting input until validated.
    while not Session(account_name=account_name, access_key=access_key).validate():
        click.echo('Authentication failed. Please try again')
        account_name = click.prompt('Account Name')
        access_key = click.prompt('Access_key')

    # Write Config
    credential_path = os.path.expanduser('~') + '/.spb/credentials'
    credential_path = credential_path.replace(os.sep, '/')

    ret = {
        'account_name': account_name,
        'access_key': access_key,
    }

    config = configparser.ConfigParser()
    config[profile_name] = ret

    with open(credential_path, 'w') as f:
        config.write(f)

@cli.command()
def project():
    helper.describe_projects()


@cli.command()
@click.argument('directory_path')
@click.option('--project_name')
def init(directory_path, project_name):
    for _ in range(3):
        project_name = click.prompt('Project Name')
        if not helper.check_project(project_name):
            click.echo('No such project. Try again.')
            project_name = None
        else:
            break
    if not project_name:
        return
    helper.init_project(directory_path, project_name)


@cli.command()
@click.argument('dataset_name')
@click.option('--log', 'log_file', type=click.File('a'))
@click.option('-f', '--force', 'is_force', default=False, is_flag=True)
def upload(dataset_name, log_file, is_force):
    helper.upload(dataset_name, log_file, is_force)


@cli.command()
@click.option('--log', 'log_file', type=click.File('a'))
def download(log_file):
    helper.download(log_file)


@cli.command()
def version():
    click.echo(spb.__version__)



import json
import os
import requests
import base64
from spb.datauri import DataURI
from spb.orm.type import Type
from spb.orm import Model
from spb.orm import AttributeModel
from spb.orm.utils import is_data_url, is_url
from spb.orm.type import String, ID, Number, Object
from spb.exceptions.exceptions import ImageIsNotExistsException


class Data(Model):
    MODEL_UUID = '0a8bb642-958b-11ea-bb37-0242ac130002'
    RESOURCE_NAME = 'asset'

    id = ID(property_name='id', immutable=True, filterable=True)
    file = String(property_name='file')
    file_name = String(property_name='fileName')
    file_size = Number(property_name='fileSize')
    dataset = String(property_name='dataset')
    data_key = String(property_name='dataKey')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if len(args) == 1 and isinstance(args[0], dict):
            # if args has dict in first row of args -> kwargs
            kwargs = args[0]
        if 'file' in kwargs:
            setattr(self, 'file', kwargs['file'])

    def __setattr__(self, name, value):
        if name is 'file':
            file_contents = None
            file_size = None
            if value is None:
                pass
            elif os.path.exists(value):
                file_size = os.path.getsize(value)
                file_name = os.path.basename(value)
                file_contents = DataURI.from_file(value)
                file_contents = file_contents.replace('\n', '') # I don't know why
                super().__setattr__('file_name', file_name)
            elif is_data_url(value):
                #TODO
                file_contents = DataURI(value)
                file_size = 0
            elif is_url(value):
                reshead = requests.head(value)
                response = requests.get(value)
                content_type = response.headers["content-type"]
                encoded_body = base64.b64encode(response.content)
                file_size = reshead.headers['Content-length']
                file_contents = f"data:{content_type};base64,{encoded_body.decode()}"
            super().__setattr__('file_size', file_size)
            super().__setattr__('file', file_contents)
        else:
            super().__setattr__(name, value)







import os
import configparser
import requests
import json
import copy
import base64

from spb.command import Command
from spb.models.project import Project
from spb.exceptions.exceptions import APIException, SDKInitiationFailedException, AuthenticateFailedException, APILimitExceededException, APIUnknowException

class Session:
    endpoint = "https://api.dev.superb-ai.com/graphql"
    headers = {
        'content-type': 'application/json',
        'cache-control': 'no-cache',
        'X-API-KEY': None,
        'Authorization': None
    }

    def __init__(self, profile=None, account_name=None, access_key=None):
        self.credential = None
        self._set_credential(
            profile=profile, account_name=account_name, access_key=access_key)

    def _set_credential(self, profile=None, account_name=None, access_key=None):
        if not profile and not account_name and not access_key:
            profile = 'default'

        if profile:
            credential_path = os.path.expanduser('~') + '/.spb/credentials'
            credential_path = credential_path.replace(os.sep, '/')
            # check exists credentials
            assert os.path.exists(credential_path), SDKInitiationFailedException(
                '** [ERROR] credentials file does not exists')
            config = self._read_config(credential_path=credential_path,
                                       profile=profile)  # get values from credential
            self.credential = config

        else:
            if account_name is None:
                raise SDKInitiationFailedException(
                    '** [ERROR] credential [account_name] does not exists')
            if access_key is None:
                raise SDKInitiationFailedException(
                    '** [ERROR] credential [access_key] does not exists')

            self.credential = {
                'account_name': account_name,
                'access_key': access_key,
            }
        self.headers['X-API-KEY'] = self.credential['access_key']
        authorization_string = base64.b64encode(f'{self.credential["account_name"]}:{self.credential["access_key"]}'.encode("UTF-8"))
        self.headers['Authorization'] = f'Basic {authorization_string.decode("UTF-8")}'

    def _read_config(self, credential_path, profile):
        config = configparser.ConfigParser()
        config.read(credential_path)
        ret = {}
        vars = ['account_name', 'access_key']
        for var in vars:
            try:
                ret[var] = config.get(profile, var)
            except (configparser.NoSectionError, configparser.NoOptionError):
                raise SDKInitiationFailedException(
                    '** [ERROR] credential - key [{0}] does not exists'.format(var))
        return ret

    def _write_config(self, credential_path, profile, ret):
        config = configparser.ConfigParser()
        config.set(profile, 'account_name', ret['account_name'])
        config.set(profile, 'access_key', ret['access_key'])
        config.write(credential_path)

    def validate(self):
        data = {
            'query': '{projects{id}}',
            'variables': {}
        }
        data = json.dumps(data)

        try:
            response = requests.request(
                "POST", self.endpoint, data=data, headers=self.headers)
        except requests.exceptions.Timeout:
            raise APIException('Occurred Time out of this request')
        except requests.exceptions.RequestException:
            raise APIException('Network Error')
        except Exception:
            raise APIException('Unknown Error on requests')

        try:
            response = response.json()
        except:
            raise APIException(
                "Failed to parse response as JSON: %s", response.text)
        errors = response.get('errors', [])
        if 'message' in response and response.get('message', None) == 'Forbidden':
            return False
        elif 'message' in response and response.get('message', None) == 'Limit Exceeded':
            raise APILimitExceededException()
        elif 'message' in response:
            raise APIUnknowException(response.get('message', None))
        if errors:
            raise APIException(errors[0]['message'])
        if 'data' not in response:
            return False
        return True

    def execute(self, model, query):
        data = {
            'query': query,
            'variables': {}
        }
        data = json.dumps(data)
        try:
            response = requests.request(
                "POST", self.endpoint, data=data, headers=self.headers)
        except requests.exceptions.Timeout:
            raise APIException('Occurred Time out of this request')
        except requests.exceptions.RequestException:
            raise APIException('Network Error')
        except Exception:
            raise APIException('Unknown Error on requests')

        try:
            response = response.json()
        except:
            raise APIException(
                "Failed to parse response as JSON: %s", response.text)
        errors = response.get('errors', [])
        # TODO: Error Handling from Server
        if 'message' in response and response.get('message', None) == 'Forbidden':
            raise AuthenticateFailedException('Check your credentials')
        elif 'message' in response and response.get('message', None) == 'Limit Exceeded':
            raise APILimitExceededException()
        elif 'message' in response:
            raise APIUnknowException(response.get('message', None))
        if errors:
            raise APIException(errors[0]['message'])
        res_data = response.get('data')
        return self._make_model_from_res(data=res_data, model=model)


    def _make_model_from_res(self, data, model):
        if data is None:
            return None
        json_datas = None
        json_datas = data[list(data.keys())[0]]
        if json_datas is None:
            raise APIException('Response doesnt have any data')
        if isinstance(json_datas, list):
            result = []
            for json in json_datas:
                temp = self._json_to_model(model=model, args=json)
                result.append(temp)
            return result
        else:
            return self._json_to_model(model=model, args=json_datas)

    def _json_to_model(self, model, args):
        return model.res_to_model(args)



import os
import logging
import logging.config
from logging.handlers import TimedRotatingFileHandler



class LoggerConfig():
  def __init__(self, logtype='STREAM', loglevel=logging.ERROR, filename='temp.log', rollover='m', interval=10, backupcount=10, format=None):
    self.logtype = logtype
    self.loglevel = loglevel
    self.filename = filename
    self.rollover = rollover
    self.interval = interval
    self.backupcount = backupcount

    if format is None:
        self.format = logging.Formatter('[%(asctime)s] %(process)d %(levelname)s in %(module)s: %(message)s')
    else:
        self.format = format

def _file_log_handler(config):
    logpath = os.path.dirname(__file__) + '/logs/' + config.filename
    logdir = os.path.dirname(logpath)
    if not os.path.exists(logdir):
        os.makedirs(logdir)
    fhandler = TimedRotatingFileHandler(logpath, when=config.rollover, interval=config.interval, backupCount=config.backupcount)
    fhandler.setFormatter(config.format)
    return fhandler

def _stream_log_handler(config):
    shandler = logging.StreamHandler()
    shandler.setFormatter(config.format)
    return shandler

def create_logger(config=LoggerConfig(), logger=None):
    if logger is not None:
        logger = logging.getLogger()
        logger.propagate = True
        logger.setLevel(config.loglevel)
        if config.logtype == 'FILE':
            handler = _file_log_handler(config)
            logger.addHandler(handler)
        elif config.logtype == 'STREAM':
            handler = _stream_log_handler(config)
            logger.addHandler(handler)
        else:
            shandler = _stream_log_handler(config)
            fhandler = _file_log_handler(config)
            logger.addHandler(shandler)
            logger.addHandler(fhandler)
    else:
        pass


import os
import click
import rich
import rich.table
import rich.console

import spb
from spb.cli_core.utils import get_project_config

console = rich.console.Console()

class Project():
    def describe_projects(self):
        projects = self._get_projects()

        table = rich.table.Table(show_header=True, header_style="bold magenta")
        table.add_column("NAME", width=20)
        table.add_column("LABELS")
        table.add_column("PROGRESS", justify="right")

        for item in projects:
            table.add_row(item.name, f"{item.label_count}", '???')

        console.print(table)

    def check_project(self, project_name):
        projects = self._get_projects()
        for project in projects:
            if project.name == project_name:
                return True
        return False

    def init_project(self, directory_path, project_name):
        if os.path.isdir(directory_path):
            console.print(f"Error whilte initiating project. directory already exists. Try again")
            return
        os.mkdir(directory_path)

        projects = self._get_projects()
        project_id = None
        for project in projects:
            if project.name == project_name:
                project_id = project.id
                break
        if project_id is not None:
            f = open(f"{directory_path}/.workspace", 'w')
            f.write(f"{project_name}\t{project_id}")
            f.close()
            console.print(f"Workspace '{directory_path}' for project '{project_name}' has been created.")
        else:
            console.print(f"Error while loading project definition. Check project name.")
            return

    def _get_projects(self):
        spb.client()
        command = spb.Command(type='describe_project')
        return spb.run(command=command)



import logging

class CustomBaseException(Exception):
    def __init__(self, message, code=None):
        super(CustomBaseException, self).__init__()
        self.logger = logging.getLogger(__name__)
        self.message = message
        self.code = code


class SDKException(CustomBaseException):
    def __init__(self, message, code='200400'):
        super(SDKException, self).__init__(message, code)


class APIException(CustomBaseException):
    def __init__(self, message, code='200500'):
        super(APIException, self).__init__(message, code)
        if message != 'The data key is duplicated.':
          self.logger.error(message, code)


class AuthenticateFailedException(APIException):
  def __init__(self, message, code='200001'):
    super().__init__(message, code)
    self.logger.error(message, code)

class APILimitExceededException(APIException):
  def __init__(self, message ='API Call limit exceeded.', code='200002'):
    super().__init__(message, code)
    self.logger.error(message, code)

class APIUnknowException(APIException):
  def __init__(self, message, code='200003'):
    super().__init__(message, code)
    self.logger.error(message, code)


class ClientCustomException(SDKException):
  def __init__(self, message, code = '000000'):
    super().__init__(message, code)


class CustomAPIException(APIException):
  def __init__(self, message, code='000001'):
    super().__init__(message, code)


class ImmutableValueChangeException(SDKException):
  def __init__(self, message, code = '100000'):
    super().__init__(message, code)
    self.logger.error(message, code)


class AttribureTypeException(SDKException):
  def __init__(self, message, code = '100001'):
    super().__init__(message, code)
    self.logger.error(message, code)


class CommandInitiationFailedException(SDKException):
  def __init__(self, message, code = '100002'):
    super().__init__(message, code)
    self.logger.error(message, code)


class ResourceIsNotExistedException(SDKException):
  def __init__(self, message, code = '100003'):
    super().__init__(message, code)
    self.logger.error(message, code)


class ModelInitiationFailedException(SDKException):
  def __init__(self, message, code = '100004'):
    super().__init__(message, code)
    self.logger.error(message, code)


class SDKInitiationFailedException(SDKException):
  def __init__(self, message, code = '100005'):
    super().__init__(message, code)
    self.logger.error(message, code)

class ImageIsNotExistsException(SDKException):
  def __init__(self, message, code = '100006'):
    super().__init__(message, code)
    self.logger.error(message, code)


import os
import rich
import rich.table
import rich.console

from spb.cli_core.commands.project import Project
from spb.cli_core.commands.label_data import LabelData
from spb.cli_core.utils import get_project_config
console = rich.console.Console()

class Helper:
    project = Project()
    label_data = LabelData()

    def set_config(self):
        pass

    def init_project(self, directory_path, project_name):
        return self.project.init_project(directory_path, project_name)

    def describe_projects(self):
        self.project.describe_projects()

    def upload(self, dataset_name, log_file, is_force):
        project_config = self._get_workspace_conf()
        return self.label_data.upload_data_and_label(project_config, dataset_name, log_file, is_force)

    def download(self, log_file):
        project_config = self._get_workspace_conf()
        return self.label_data.download_data_and_label(project_config, log_file)

    def check_project(self, project_name):
        return self.project.check_project(project_name)

    def _get_workspace_conf(self):
        workspace_conf_path = '.workspace'
        if os.path.isfile(workspace_conf_path):
            try:
                f = open(f"{workspace_conf_path}", 'r')
                return get_project_config(f.readline())
            except:
                console.print(f"Error while reading workspace configuration. Try again")
                return None
        else:
            console.print(f"Workspace is not initiated. 'spb init' first.")
            return None


# MIT License
#
# Copyright (c) 2020 Superb AI Corporation.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from .commands import (
    CreateCommand,
    DescribeCommand,
    DeleteCommand,
    UpdateCommand
)
from spb.exceptions.exceptions import CommandInitiationFailedException


class CommandBuilder:
    """

    """
    __command_map_ = {
        'describe': DescribeCommand,
        'create': CreateCommand,
        'delete': DeleteCommand,
        'update': UpdateCommand
    }

    def __init__(self, op_name, res_name):
        self.op_name = op_name
        self.res_name = res_name

    def build(self):
        if not self.op_name:
            raise CommandInitiationFailedException('Command operation name is not correct')

        if not self.res_name:
            raise CommandInitiationFailedException('Command resource name is not correct')

        return self.__command_map_[self.op_name](self.res_name)


class CmdMeta(type):
    """
    Command Metaclass 
    """

    def __call__(self, *args, **kwargs):
        str_cmd = kwargs.get('type', None)
        if not str_cmd:
            raise CommandInitiationFailedException('Command need to init with type param')

        op_name, res_name = str_cmd.split('_')
        if not op_name or not res_name:
            raise CommandInitiationFailedException('Command need to take the form of "[op_name]_[resource_name]"')

        return CommandBuilder(op_name, res_name).build()


class Command(metaclass=CmdMeta):
    def __init__(self, type):
        print(self.__dict__)



import copy
import json
from spb.orm.manager import Manager
from spb.orm.type import *
from spb.orm.utils import *
from spb.exceptions.exceptions import ModelInitiationFailedException

class ModelMeta(type):
    def __new__(cls, name, bases, attrs, **kwargs):
        super_new = super(ModelMeta, cls).__new__
        parents = [b for b in bases if isinstance(b, ModelMeta)]
        if not parents:
            # If this isn't a subclass of Model, don't do anything special.
            return super_new(cls, name, bases, attrs)

        module = attrs.pop('__module__')

        new_attrs = {'__module__': module}
        new_attrs.update(attrs)

        model_attrs = {}
        for obj_name, obj in attrs.items():
            if isinstance(obj, Type):
                model_attrs[obj_name] = obj

        attr_classes = {}
        for obj_name, obj in attrs.items():
            if isinstance(obj, Type):
                attr_classes[obj_name] = obj.__class__

        graphql_to_python_attr = {}
        for obj_name, obj in attrs.items():
            if isinstance(obj, Type):
                graphql_to_python_attr[obj.attr_name] = obj_name

        immutable_attr = []
        for obj_name, obj in attrs.items():
            if isinstance(obj, Type) and obj.immutable:
                immutable_attr.append(obj.attr_name)

        filterable_attr = []
        for obj_name, obj in attrs.items():
            if isinstance(obj, Type) and obj.filterable:
                filterable_attr.append(obj.attr_name)

        instance_attr = {}
        for obj_name, obj in attrs.items():
            if isinstance(obj, Type) and obj.express:
                instance_attr[obj_name] = obj.express
            elif isinstance(obj, Type) and isinstance(obj, JsonObject):
                instance_attr[obj_name] = dict

        new_attrs['_immutables'] = immutable_attr
        new_attrs['_filterables'] = filterable_attr
        new_attrs['_instances'] = instance_attr
        new_attrs['_attrs_classes'] = attr_classes

        # make class with attributes
        new_class = super_new(cls, name, bases, new_attrs, **kwargs)
        # insert _attrs to class for type validation
        new_class.add_to_class('_attrs', model_attrs)
        new_class.add_to_class('_graphql_attrs', graphql_to_python_attr)

        return new_class

    def add_to_class(cls, name, value):
        setattr(cls, name, value)


class AttributeContainer:
    def __init__(self, attrs, kwargs=None):
        if len(kwargs) > len(attrs):
            # check argumets numbers over
            raise ModelInitiationFailedException("Number of args exceeds number of parameters")

        self.attribute_values = {}
        for key in attrs:
            # TODO: set default values
            self.attribute_values[key] = None
        for key in kwargs.keys():
            if key in self._instances:
                if is_json(kwargs[key]):
                    kwargs[key] = json.loads(kwargs[key])
                if isinstance(kwargs[key], dict) and not issubclass(self._instances[key], ListAttribute):
                    kwargs[key] = self._instances[key](**kwargs[key])
                elif isinstance(kwargs[key], dict):
                    kwargs[key] = [self._instances[key](**kwargs[key])]
                elif isinstance(kwargs[key], list) and not issubclass(self._instances[key], ListAttribute):
                    kwargs[key] = self._instances[key](**kwargs[key])
                elif isinstance(kwargs[key], list):
                    kwargs[key] = [self._instances[key](**item) for item in kwargs[key]]
            self.attribute_values[key] = kwargs[key]

    def _set_default(self, attr):
        pass

    @classmethod
    def res_to_model(cls, inputs):
        print(inputs)
        gql_attrs = cls.get_attr_names()
        transformed_attrs = {}
        for key, attr_name in gql_attrs.items():
            if key not in inputs:
                raise ModelInitiationFailedException('Response data is not matched requested resource')
            value = inputs[key]
            transformed_attrs[attr_name] = value
        result = cls(**transformed_attrs)
        return result


class Model(AttributeContainer, metaclass=ModelMeta):  # pylint: disable=R0205
    ''' Abstract entity model with an active record interface '''
    RESOURCE_NAME = 'None'

    def __init__(self, *args, **kwargs):
        cls = self.__class__
        attrs = cls._attrs
        if len(args) == 1 and isinstance(args[0], dict):
            # if args has dict in first row of args -> kwargs
            kwargs = args[0]
        elif len(args) != 0:
            raise ModelInitiationFailedException("Args can only be a dictionary")

        super().__init__(attrs=attrs, kwargs=kwargs)

    @property
    def public(self):
        ''' Return the public model attributes '''
        return attrs(self)

    def __repr__(self):
        return str(self.public)

    def get_resource_name(self):
        return self.RESOURCE_NAME

    @classmethod
    def manager(cls, type_check=True):
        ''' Create a database managet '''
        return Manager(cls, type_check)

    @classmethod
    def get_attr_names(cls):
        return cls._graphql_attrs

class AttributeModel:
    def get_datas(self, item):
        attrs = self.__dict__
        attrs_dict = {}
        for attr in attrs.keys():
            value = getattr(self, attr, None)
            value_string = None
            if not isinstance(value, AttributeModel) and not isinstance(value, list):
                value_string = value
            else:
                if isinstance(value, list):
                    result_items = []
                    for item in value:
                        if isinstance(item, AttributeModel):
                            result_items.append(item.get_datas(item))
                        else:
                            result_items.append(item)
                    value_string = result_items
                else:
                    value_string = value.get_datas(value)
            attrs_dict[attr] = value_string
        return attrs_dict

class ListAttribute:
    pass



import os
import click
import urllib
import json
import rich
import rich.table
import rich.console
from multiprocessing import Process, Manager, Pool
import tqdm

import spb
from spb.cli_core.utils import recursive_glob_image_files
from spb.models.label import Label

console = rich.console.Console()

NUM_MULTI_PROCESS = 4
LABEL_DESCRIBE_PAGE_SIZE = 10

class LabelData():
    def upload_data_and_label(self, project_config, dataset_name, log_file, is_force):
        project_id = project_config[1]

        imgs_path = recursive_glob_image_files(dataset_name)
        if not click.confirm(f"Upload {len(imgs_path)} files to project '{project_config[0]}'. Proceed?"):
            return
        asset_images = []
        for key in imgs_path:
            file_name = key
            asset_image = {
                'file': imgs_path[key],
                'file_name': file_name,
                'data_key': key,
                'dataset': dataset_name
            }
            asset_images.append(asset_image)

        with Pool(NUM_MULTI_PROCESS) as p:
            list(tqdm.tqdm(p.imap(_upload_asset, zip([project_id] * len(asset_images), asset_images)), total=len(asset_images), desc="Uploading Data"))

        with Pool(NUM_MULTI_PROCESS) as p:
            labels = list(p.imap(_describe_label_by_asset, zip([project_id] * len(asset_images), asset_images)))

        if sum(label['result'] is not None for label in labels) > 0:
            if not is_force and not click.confirm(f"{sum(label['result'] is not None for label in labels)} files are already labeled. Overwrite?"):
                return

        with Pool(NUM_MULTI_PROCESS) as p:
            list(tqdm.tqdm(p.imap(_update_label, zip([project_id] * len(labels), labels)), total=len(labels), desc="Uploading labels"))


    def download_data_and_label(self, project_config, log_file):
        project_id = project_config[1]
        command = spb.Command(type='describe_project')
        projects = spb.run(command=command, option={"id":project_id})
        label_count = projects[0].label_count
        page_length = int(label_count/LABEL_DESCRIBE_PAGE_SIZE) if label_count % LABEL_DESCRIBE_PAGE_SIZE == 0 else int(label_count/LABEL_DESCRIBE_PAGE_SIZE)+1

        with Pool(NUM_MULTI_PROCESS) as p:
            labels_list = list(p.imap(_describe_label_by_page, zip([project_id] * page_length, range(page_length))))
            labels = [item for sublist in labels_list for item in sublist]

        if not click.confirm(f"Download {len(labels)} files from project '{project_config[0]}'. Proceed?"):
            return

        with Pool(NUM_MULTI_PROCESS) as p:
            for _ in tqdm.tqdm(p.imap(_download_label_data, labels), total=len(labels)):
                pass

def _describe_label_by_page(args):
    [project_id, page_idx] = args
    command = spb.Command(type='describe_label')
    labels = spb.run(command=command, option={
        'project_id' : project_id
    }, page_size = LABEL_DESCRIBE_PAGE_SIZE, page = page_idx + 1)
    return labels

def _download_label_data(label):
    data_url = label.data_url
    path = label.dataset + label.data_key if label.data_key.find('/') != -1 else label.dataset + "/"+label.data_key
    directory_name = os.path.dirname(path)
    if not os.path.isdir(directory_name):
        os.mkdir(directory_name)
    urllib.request.urlretrieve(data_url, path)

    f = open(path + ".json", 'w')
    f.write(label.toJson())
    f.close()

def _upload_asset(args):
    [project_id, asset_image] = args
    try:
        command = spb.Command(type='create_data')
        spb.run(command=command, option=asset_image, optional={'projectId': project_id})

    except Exception:
        pass

def _describe_label_by_asset(args):
    [project_id, asset_image] = args

    data_key = asset_image['data_key']
    dataset = asset_image['dataset']
    command = spb.Command(type='describe_label')
    option = {
        "project_id": project_id,
        "data_key": data_key,
        "dataset": dataset
    }

    labels = spb.run(command=command, option=option, page_size=1, page=1)
    label = labels[0]

    return {
        "id": label.id,
        "project_id": label.project_id,
        "data_key": label.data_key,
        "dataset": label.dataset,
        "result": label.result,
    }

def _update_label(args):
    [project_id, label] = args

    data_key = label['data_key']
    dataset = label['dataset']

    json_path = f"{dataset}/{data_key}.json"

    if not os.path.isfile(json_path):
        return

    try:
        with open(json_path) as json_file:
            json_data = json.load(json_file)

        if json_data['result'] == None:
            return
        label['result'] = json_data['result']

        command = spb.Command(type='update_label')
        label = spb.run(command=command, option=label)

        with open(json_path, 'w') as f:
           f.write(label.toJson())

    except Exception as err:
        console.print(err)
        console.print(f"Error while uploading label '{label['data_key']}.json'. Try again.")



# MIT License
#
# Copyright (c) 2020 Superb AI Corporation.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# It is comming from Django loading code snippet. So, if you want to look into details
# please visit to

import sys
import os
import glob
import importlib
import threading
from spb.orm.model import Model
from spb.exceptions.exceptions import ResourceIsNotExistedException


class Loader(object):
    """
    A cache that stores installed applications and their models. Used to
    provide reverse-relations and for app introspection (e.g. admin).
    """
    # Use the Borg pattern to share state between all instances. Details at
    # http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66531.
    __model_classes_ = dict(
        loaded=False,
        write_lock=threading.RLock(),
        models={}
    )

    def __init__(self):
        self.__dict__ = self.__model_classes_

    def _populate(self):
        """
        Fill in all the cache information. This method is threadsafe, in the
        sense that every caller will see the same state upon return, and if the
        cache is already initialised, it does no work.
        """
        if self.loaded:
            return
        self.write_lock.acquire()
        try:
            if self.loaded:
                return
            self.load_model()
            self.loaded = True
        finally:
            self.write_lock.release()

    def get_model(self, res_model):
        """
        Given a module containing models, returns a list of the models.
        Otherwise returns a list of all installed models.
        """
        if not self.loaded:
            self._populate()

        return self.models[res_model]

    def load_model(self):
        BASE_DIR = "/".join(os.path.dirname(os.path.abspath(__file__)
                                            ).replace(os.sep, '/').split('/')[0:-1])+'/models/'
        file_list = glob.glob(BASE_DIR + '*.py')
        for module in file_list:
            if module.find('__init__') == -1:
                # module_file = os.path.join(BASE_DIR, module)
                module_file = module.replace(os.sep, '/')
                # TODO:// object -> Base class
                inherited_class = self._import_class(module_file, Model)
                self.models[inherited_class.__name__.lower()] = inherited_class

    def _import_class(self, implementation_filename, base_class):
        impl_dir, impl_filename = os.path.split(implementation_filename)
        module_name, _ = os.path.splitext(impl_filename)

        try:
            sys.path.insert(0, impl_dir)
            module = importlib.import_module(module_name)
            for name in dir(module):
                obj = getattr(module, name)
                if (type(obj) == type(base_class)
                    and issubclass(obj, base_class)
                        and obj != base_class):
                    return obj
            raise ResourceIsNotExistedException(
                f"No subclass of {base_class.__name__} in {implementation_filename}")
        finally:
            sys.path.pop(0)



# MIT License
#
# Copyright (c) 2020 Superb AI Corporation.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import abc

from spb.orm import Loader


class BaseCommand(metaclass=abc.ABCMeta):
    """
    A BaseCommand is base class of command operation classes.
    """

    def __init__(self, res_name):
        """
        type: str
        res_name: resource name 
        """
        self.res_name = res_name
        self._loader = Loader()

    @abc.abstractmethod
    def execute(self, session=None, option=None, optional=None):
        """
        Caution: 
        """
        if type(self) is BaseCommand:
            raise NotImplementedError("Interfaces can't be instantiated")


class DescribeCommand(BaseCommand):
    """
    Describe command is for retrieving
    """

    def execute(self, session=None, option=None, optional=None):
        DescribeModel = self._loader.get_model(self.res_name)
        return DescribeModel.manager().query(option, optional)


class CreateCommand(BaseCommand):
    """
    Creation command
    """

    def execute(self, session=None, option=None, optional=None):
        CreateModel = self._loader.get_model(self.res_name)
        return CreateModel.manager().mutation(option, optional)


class DeleteCommand(BaseCommand):
    """
    Delete command
    """

    def execute(self, session=None, option=None, optional=None):
        print('exeute in DeleteCommand')


class UpdateCommand(BaseCommand):
    """
    Update command
    """

    def execute(self, session=None, option=None, optional=None):
        UpdateModel = self._loader.get_model(self.res_name)
        return UpdateModel.manager().mutation(option, optional)



import abc
import json
from uuid import UUID
from spb.exceptions.exceptions import ImmutableValueChangeException, AttribureTypeException


class Type(object):
    type_name = None

    def __init__(self, *args, **kwargs):
        self._filterable = False
        self._immutable = False
        self._express = None

        # self.attr = kwargs['property_name'] if 'property_name' in kwargs else None
        self.attr = kwargs['property_name']

        if kwargs is not None and 'immutable' in kwargs:
            self._immutable = kwargs['immutable']
        if kwargs is not None and 'filterable' in kwargs:
            self._filterable = kwargs['filterable']

        if kwargs is not None and 'express' in kwargs:
            self._express = kwargs['express']

    @property
    def attr_name(self):
        return self.attr

    @attr_name.setter
    def attr_name(self, value):
        self.attr = value

    @property
    def filterable(self):
        return self._filterable

    @filterable.setter
    def filterable(self, value):
        self._filterable = value

    @property
    def immutable(self):
        return self._immutable

    @immutable.setter
    def immutable(self, value):
        self._immutable = value

    @property
    def express(self):
        return self._express

    @express.setter
    def express(self, value):
        self._express = value

    @abc.abstractmethod
    def _validation(self, value):
        ''' Verify value attribute according to user defined code '''
        if self._immutable:
            raise ImmutableValueChangeException(
                'This is immutable value. Check the model attribute type')
        return value

    @abc.abstractmethod
    def _serialize(self, value):
        return value

    def __set__(self, instance, value):
        attribute_name = instance._graphql_attrs[self.attr]
        if self._express is not None and isinstance(value, dict):
            value = self._express(value)

        value = self._validation(value)
        instance.attribute_values[attribute_name] = str(value)

    def __get__(self, instance, objtype):
        attribute_name = instance._graphql_attrs[self.attr]
        value = instance.attribute_values[attribute_name]
        value = self._serialize(value)
        return value

    def __repr__(self):
        return str(self.type_name)

    @classmethod
    def get_attr_name(cls):
        return cls.attr_name

    def __eq__(self, you):
        if isinstance(you, Type):
            return str(self.value) == str(you.value)
        else:
            return str(self.value) == str(you)


class Number(Type):
    def _validation(self, value):
        if self._immutable:
            raise ImmutableValueChangeException(f"'{self.attr_name}' cannot be changed")

        if value is not None and not str(value).isdigit():
            raise AttribureTypeException(f"'{self.attr_name}' need number type value")

        return value

    @classmethod
    def _type_to_query(cls, name, value):
        query = {}
        query[name] = value
        return query


class ID(Type):
    def _validation(self, value):
        if self._immutable:
            raise ImmutableValueChangeException(f"'{self.attr_name}' cannot be changed")
        try:
            if value is not None:
                uuid_result = UUID(value, version=4)
                if value != uuid_result:
                    raise ValueError
        except ValueError:
            raise AttribureTypeException(f"'ID' can be UUID4 string")

        return value

    @classmethod
    def _type_to_query(cls, name, value):
        query = {}
        query[name] = f'"{value}"'
        return query


class String(Type):
    def _validation(self, value):
        if self._immutable:
            raise ImmutableValueChangeException(f"'{self.attr_name}' cannot be changed")
        if value is not None and not isinstance(value, str):
            raise AttribureTypeException(f"'{self.attr_name}' need string type value")

        return value

    @classmethod
    def _type_to_query(cls, name, value):
        query = {}
        query[name] = f'"{value}"'
        return query


class Boolean(Type):
    def _validation(self, value):
        if self._immutable:
            raise ImmutableValueChangeException(f"'{self.attr_name}' cannot be changed")

        if value is not None and not type(value) == bool:
            raise AttribureTypeException(f"'{self.attr_name}' need boolean type value")

        return value

    @classmethod
    def _type_to_query(cls, name, value):
        query = {}
        query[name] = value
        return query


class Object(Type):
    @classmethod
    def _type_to_query(cls, name, value):
        query = {}
        if isinstance(value, list):
            result = [item.get_datas(item) for item in value]
        else:
            result = value.get_datas(value)
        json_result = json.dumps(result)
        json_result = json_result.replace("\"","\\\"")
        query[name] = f'"{json_result}"'
        return query

class JsonObject(Type):
    def _validation(self, value):
        if self._immutable:
            raise ImmutableValueChangeException(f"'{self.attr_name}' cannot be changed")
        if value is not None and not type(value) == dict:
            raise AttribureTypeException(f"'{self.attr_name}' need dictionary type value")
        return value
    def _serialize(self, value):
        if value == 'None' or value is None:
            return None
        elif isinstance(value, dict):
            return value

        value = value.replace("'", '"')
        value = value.replace("None", "null")
        value = json.loads(str(value))
        return value
    @classmethod
    def _type_to_query(cls, name, value):
        query = {}
        if value == 'None' or value == None:
            query[name] = 'null'
            return query

        value = json.dumps(value)
        # value = value.replace("\"","\\\"")
        # value = value.replace("\"\"", "\"")
        value = value.replace('None', 'null')
        if value.find('\'') != -1:
            value = value.replace("\'", "\\\"")
            query[name] = f'{value}'
        else:
            value = value.replace("\"", "\\\"")
            query[name] = f'"{value}"'
        return query


import json

from spb.orm.type import Type
from spb.orm import Model
from spb.orm import AttributeModel
from spb.orm.type import String, ID, Number, Object

class Class(AttributeModel):
    def __init__(self, *args, **kwargs ):
        self.name = kwargs['name'] if 'name' in kwargs else None
        self.annotationType = kwargs['annotationType'] if 'annotationType' in kwargs else None
        self.properties = kwargs['properties'] if 'properties' in kwargs else None


class Configure(AttributeModel):
    def __init__(self, *args, **kwargs):
        self.dataType = kwargs['dataType'] if 'dataType' in kwargs else None
        self.annotationTypes = kwargs['annotationTypes'] if 'annotationTypes' in kwargs else None
        self.classList = [Class(**item) for item in kwargs['classList']] if 'classList' in kwargs else None


class KeypointInterface(AttributeModel):
    def __init__(self, *args, **kwargs):
        self.name = kwargs['name'] if 'name' in kwargs else None
        self.points = kwargs['points'] if 'points' in kwargs else None


class ObjectClass(AttributeModel):
    def __init__(self, *args, **kwargs):
        self.name = kwargs['name'] if 'name' in kwargs else None
        self.color = kwargs['color'] if 'color' in kwargs else None
        self.annotationType = kwargs['annotationType'] if 'annotationType' in kwargs else None
        # optional value : annotation_type = keypoint
        self.keypointName = kwargs['keypointName'] if 'keypointName' in kwargs else None


class ObjectGroups(AttributeModel):
    def __init__(self, *args, **kwargs):
        self.name = kwargs['name'] if 'name' in kwargs else None
        self.objectClasses = kwargs['objectClasses'] if 'objectClasses' in kwargs else None


class ObjectDetection(AttributeModel):
    def __init__(self, *args, **kwargs):
        self.annotationTypes = kwargs['annotationTypes'] if 'annotationTypes' in kwargs else []
        self.objectClasses = [ObjectClass(
            **item) for item in kwargs['objectClasses']] if 'objectClasses' in kwargs else []
        self.objectGroups = [ObjectGroups(
            **item) for item in kwargs['objectGroups']] if 'objectGroups' in kwargs else []
        self.keypointInterfaces = [KeypointInterface(
            **item) for item in kwargs['keypointInterfaces']] if 'keypointInterfaces' in kwargs else []


class ImageCategorization(AttributeModel):
    def __init__(self, *args, **kwargs):
        self.name = kwargs['name'] if 'name' in kwargs else None
        self.type = kwargs['type'] if 'type' in kwargs else None
        self.options = kwargs['options'] if 'options' in kwargs else []


class ProjectSummary(AttributeModel):
    def __init__(self, *args, **kwargs):
        self.dataType = kwargs['dataType'] if 'dataType' in kwargs else None
        self.objectDetection = ObjectDetection(
            **kwargs['objectDetection']) if 'objectDetection' in kwargs else ObjectDetection()
        self.imageCategorization = [ImageCategorization(
            **item) for item in kwargs['imageCategorization']] if 'imageCategorization' in kwargs else []


class Project(Model):
    MODEL_UUID = '1da666a0-958b-11ea-bb37-0242ac130002'
    RESOURCE_NAME = 'projects'

    id = ID(property_name='id', immutable=True, filterable=True)
    name = String(property_name='name')
    label_count = Number(property_name='labelCount')
    # configure = Type(property_name='configure', express=Configure)
    summary = Object(property_name='configure', express=ProjectSummary)



# MIT License
#
# Copyright (c) 2020 Superb AI Corporation.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import logging

# import spb.exceptions

from spb.session import Session
from spb.command import Command
# from spb.spb_logger import SPBLogger
from spb.spb_logger import create_logger
from spb.exceptions.exceptions import CommandInitiationFailedException


__author__  = 'Super AI Dev Team'
__version__ = '0.0.1'


DEFAULT_SESSION = None
logger = None

__all__ = ('setup_default_session', '_get_default_session', 'client', 'run')

def setup_default_session(**kwargs):
    global DEFAULT_SESSION
    DEFAULT_SESSION = Session(**kwargs)


def _get_default_session(**kwargs):
    if DEFAULT_SESSION is None:
        setup_default_session(**kwargs)

    return DEFAULT_SESSION

def client(**kwargs):
    if 'logger_config' in kwargs:
        create_logger(config=kwargs['logger_config'])
    elif 'logger' in kwargs:
        create_logger(logger=kwargs['logger'])
    else:
        create_logger()
    return _get_default_session(**kwargs)


def run(command: Command, option: dict = {}, page: int = None, page_size: int = None, optional: dict = {}):
    if command is None:
        raise CommandInitiationFailedException('Command required argument')
    if page is not None:
        optional['page'] = page
    if page_size is not None:
        optional['pageSize'] = page_size
    return command.execute(session=DEFAULT_SESSION, option=option, optional=optional)






import re
import os
import glob
import imghdr

def get_project_config(line):
    r = re.compile(r'([^\t]*)\t*')
    return r.findall(line)

def recursive_glob_image_files(input_path):
    support_img_format = ['png', 'jpg', 'bmp', 'jpeg', 'tiff', 'tif']
    imgs_path = {}
    files_path = sorted(glob.glob(os.path.join(input_path, "**/*"), recursive=True))
    for file_path in files_path:
        if '@' not in file_path and not os.path.isdir(file_path):
            img_format = imghdr.what(file_path)
            if img_format in support_img_format:
                key = extract_file_key(input_path, file_path)
                imgs_path[key] = file_path
    return imgs_path

def extract_file_key(input_folder_path, file_path):
    key = file_path.replace(os.path.abspath(input_folder_path), "")
    key = "/".join(key.split("/")[1:])
    return key


import io
from setuptools import setup, find_packages


# Read in the README for the long description on PyPI
def long_description():
    with io.open("README.md", "r", encoding="utf-8") as f:
        readme = f.read()
    return readme


setup(
    name="cool-tree",
    version="0.0.18",
    url="https://github.com/Superb-AI-Suite/cool-tree.git",
    license="MIT",
    author="Superb AI",
    author_email="support@superb-ai.com",
    description="Suite Standard Library",
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
    ],
    packages=find_packages(),
    entry_points={"console_scripts": ["spb=spb.__main__:cli"]},
    long_description=long_description(),
    long_description_content_type="text/markdown",
    install_requires=[
        "bleach==3.1.5",
        "certifi==2020.4.5.1",
        "chardet==3.0.4",
        "colorama==0.4.3",
        "commonmark==0.9.1",
        "docutils==0.16",
        "idna==2.9",
        "importlib-metadata==1.6.0; python_version < '3.8'",
        "keyring==21.2.1",
        "packaging==20.3",
        "pkginfo==1.5.0.1",
        "pprintpp==0.4.0",
        "pygments==2.6.1",
        "pyparsing==2.4.7",
        "python-datauri==0.2.8",
        "readme-renderer==26.0",
        "requests==2.23.0",
        "requests-toolbelt==0.9.1",
        "rich==1.0.0",
        "six==1.14.0",
        "stringcase==1.2.0",
        "tqdm==4.46.0",
        "twine==3.1.1",
        "typing-extensions==3.7.4.2",
        "urllib3==1.25.9",
        "webencodings==0.5.1",
        "zipp==3.1.0",
    ],
    zip_safe=False,
    dependency_links=[],
)
